import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:geo_sdk_flutter_plugin/models/ld_event.dart';


typedef void LDEventResultHandler(LDEvent event);

typedef void PayloadResultHandler(int count);

class GeoSdkFlutterPlugin {
  static const MethodChannel _methodChannel =
      const MethodChannel('geo_sdk_flutter_plugin');

  static final GeoSdkFlutterPlugin _geoSdk = new GeoSdkFlutterPlugin._internal();

  factory GeoSdkFlutterPlugin() => _geoSdk;

  LDEventResultHandler onEventHandler;

  PayloadResultHandler onPayloadSent;


  GeoSdkFlutterPlugin._internal() {
    _methodChannel.setMethodCallHandler(_platformCallHandler);
  }



  Future _platformCallHandler(MethodCall methodCall) async {
    /// Only for debug purposes
    print('method is being called: ${methodCall.method} and with arguments: ${methodCall.arguments}');
    switch (methodCall.method) {
      case "onEvent":
        onEventHandler(methodCall.arguments);
        break;
      case "onPayloadSent":
        onPayloadSent(methodCall.arguments);
        break;
      default:
        print('GEO_SDK_FLUTTER_PLUGIN: Unknown method called - ${methodCall.method} ');
    }

  }


  /// To handle the events that are sen't from the native platform, set a handler in
  /// your code
  setOnEventHandler(LDEventResultHandler handler) => onEventHandler = handler;

  /// To know the amount of payloads that were sent, set a handler to this method
  setOnPayloadSentHandler(PayloadResultHandler handler) => onPayloadSent = handler;


  /// To start the SDK, run this method to check location permissions
  /// And connection with LOTaDATA
  Future<bool> checkPermissionsAndLaunch() async {
    final bool result = await _methodChannel.invokeMethod('checkPermissionsAndLaunch');
    return result;
  }



  ///If there's a LDEvent saved it will return it, otherwise
  /// the return will be null
  Future<LDEvent> get getLastEvent async {
    final LDEvent ldEvent = await _methodChannel.invokeMethod('getLastEvent');
    return ldEvent;
  }



  ///Use this method to be sure if you're using the latest SDK version.
  Future<String> get getSDKVersion async {
    final String version = await _methodChannel.invokeMethod('getSDKVersion');
    return version;
  }

  /// Call this method on your dart widget on dispose method before [super.dispose()]
  Future<void> dispose() async {
    await _methodChannel.invokeMethod('disposeSdk');
  }

}
